import {
  WebStorage
} from "./chunk-YWADTQF4.js";

// src/identity/create-frame-context-hook.ts
import { useCallback, useEffect, useMemo, useRef, useState } from "react";
var defaultStorage = new WebStorage();
function createFrameContextHook(options) {
  return function useFrameContext({
    fallbackContext,
    storage = defaultStorage,
    storageKey = options.storageKey
  }) {
    const storageRef = useRef(storage);
    const [frameContext, setFrameContext] = useState(
      null
    );
    useEffect(() => {
      storageRef.current.get(storageKey).then((data) => {
        if (data) {
          setFrameContext(data);
        }
      }).catch((e) => {
        console.error(
          "@frames.js/render: Could not load frame context from storage",
          e
        );
      });
    }, [storageKey]);
    const handleSetFrameContext = useCallback(
      async (newFrameContext) => {
        await storageRef.current.set(
          storageKey,
          () => newFrameContext
        );
        setFrameContext(newFrameContext);
      },
      [storageKey]
    );
    const resetFrameContext = useCallback(async () => {
      await storageRef.current.delete(storageKey);
      setFrameContext(null);
    }, [storageKey]);
    return useMemo(
      () => ({
        frameContext: frameContext || fallbackContext,
        setFrameContext: handleSetFrameContext,
        resetFrameContext
      }),
      [fallbackContext, frameContext, handleSetFrameContext, resetFrameContext]
    );
  };
}

export {
  createFrameContextHook
};
