// src/hooks/use-visibility-detection.ts
import { useEffect, useMemo, useRef } from "react";
function useVisibilityDetection() {
  const listenersRef = useRef([]);
  useEffect(() => {
    if (typeof document !== "undefined") {
      const handleVisibilityChange = () => {
        const isVisible = !document.hidden;
        listenersRef.current.forEach((fn) => {
          fn(isVisible);
        });
      };
      document.addEventListener("visibilitychange", handleVisibilityChange);
      return () => {
        document.removeEventListener(
          "visibilitychange",
          handleVisibilityChange
        );
      };
    }
  }, []);
  return useMemo(
    () => ({
      register(fn) {
        listenersRef.current.push(fn);
        return () => {
          listenersRef.current = listenersRef.current.filter(
            (listener) => listener !== fn
          );
        };
      }
    }),
    []
  );
}

export {
  useVisibilityDetection
};
