"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/collapsed-frame-ui.tsx
var collapsed_frame_ui_exports = {};
__export(collapsed_frame_ui_exports, {
  CollapsedFrameUI: () => CollapsedFrameUI
});
module.exports = __toCommonJS(collapsed_frame_ui_exports);
var import_react = __toESM(require("react"), 1);
var defaultTheme = {
  buttonBg: "#fff",
  buttonBorderColor: "#ccc",
  buttonHoverBg: "#efefef",
  buttonColor: "#444",
  buttonRadius: "4",
  bg: "#efefef"
};
var getThemeWithDefaults = (theme) => {
  return {
    ...defaultTheme,
    ...theme
  };
};
function CollapsedFrameUI({
  frameState,
  theme,
  FrameImage,
  allowPartialFrame
}) {
  const [isImageLoading, setIsImageLoading] = (0, import_react.useState)(true);
  const currentFrame = frameState.currentFrameStackItem;
  const isLoading = currentFrame?.status === "pending" || isImageLoading;
  const resolvedTheme = getThemeWithDefaults(theme ?? {});
  if (!frameState.homeframeUrl) {
    return null;
  }
  if (!currentFrame) {
    return null;
  }
  if (currentFrame.status === "done" && currentFrame.frameResult.status === "failure" && !(allowPartialFrame && // Need at least image and buttons to render a partial frame
  currentFrame.frameResult.frame.image && currentFrame.frameResult.frame.buttons)) {
    return null;
  }
  let frame;
  if (currentFrame.status === "done") {
    frame = currentFrame.frameResult.frame;
  } else if (currentFrame.status === "message" || currentFrame.status === "doneRedirect") {
    frame = currentFrame.request.sourceFrame;
  } else if (currentFrame.status === "requestError") {
    frame = "sourceFrame" in currentFrame.request ? currentFrame.request.sourceFrame : void 0;
  }
  const ImageEl = FrameImage ? FrameImage : "img";
  return /* @__PURE__ */ import_react.default.createElement(
    "div",
    {
      style: { backgroundColor: resolvedTheme.bg },
      className: "flex w-full gap-2 rounded p-2 items-center"
    },
    /* @__PURE__ */ import_react.default.createElement(
      "div",
      {
        className: "relative max-h-[60px] shrink-0",
        style: { aspectRatio: "1/1" }
      },
      " ",
      !!frame && !!frame.image && /* @__PURE__ */ import_react.default.createElement(
        ImageEl,
        {
          src: frame.image,
          key: frame.image,
          alt: "Frame image",
          width: "100%",
          style: {
            filter: isLoading ? "blur(4px)" : void 0,
            borderRadius: `${resolvedTheme.buttonRadius}px`,
            border: `1px solid ${resolvedTheme.buttonBorderColor}`,
            objectFit: "cover",
            width: "100%",
            aspectRatio: "1/1"
          },
          onLoadStart: () => {
            setIsImageLoading(true);
          },
          onLoad: () => {
            setIsImageLoading(false);
          },
          onError: () => {
            setIsImageLoading(false);
          }
        }
      )
    ),
    /* @__PURE__ */ import_react.default.createElement("div", { className: "flex flex-col flex-1 justify-center min-w-[0]" }, /* @__PURE__ */ import_react.default.createElement("span", { className: "font-semibold block truncate" }, frame?.title), /* @__PURE__ */ import_react.default.createElement("span", { className: "text-gray-500 text-xs block truncate" }, new URL(currentFrame.url).hostname)),
    !!frame && !!frame.buttons ? /* @__PURE__ */ import_react.default.createElement("div", { className: "flex items-center shrink-0" }, /* @__PURE__ */ import_react.default.createElement(
      "button",
      {
        type: "button",
        disabled: isLoading,
        className: `p-2 ${isLoading ? "bg-gray-100" : ""} border text-sm text-gray-800 rounded`,
        style: {
          flex: "1 1 0px",
          // fixme: hover style
          backgroundColor: resolvedTheme.buttonBg,
          borderColor: resolvedTheme.buttonBorderColor,
          color: resolvedTheme.buttonColor,
          cursor: isLoading ? void 0 : "pointer"
        }
      },
      frame.buttons.length === 1 && frame.buttons[0].label.length < 12 ? frame.buttons[0].label : "View"
    )) : null
  );
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  CollapsedFrameUI
});
