import { Storage } from '../identity/types.cjs';
import './use-visibility-detection.cjs';

type Primitives = string | number | boolean | null;
type Arr = AllowedValue[];
type Obj = {
    [key in string]: AllowedValue;
} & {
    [key in string]?: AllowedValue | undefined;
};
type AllowedValue = Primitives | Arr | Obj;
type SharedOptions<TValue extends AllowedValue | undefined> = {
    key: string;
    /**
     * Called each time the value in storage changes or is loaded.
     *
     * You can modify the value before it is set in the state.
     */
    preprocessValue?: (value: Exclude<TValue, undefined>) => TValue;
    /**
     * @defaultValue WebStorage
     */
    storage?: Storage;
};
type UseStorageWithoutInitialValue<TValue extends AllowedValue> = {
    initialValue?: never;
} & SharedOptions<TValue>;
type UseStorageOptionsWithInitialValue<TValue extends AllowedValue> = {
    initialValue: TValue;
} & SharedOptions<TValue>;
type Setter<TValue extends AllowedValue | undefined> = (value: TValue | ((prevState: TValue) => TValue)) => Promise<void>;
declare function useStorage<TValue extends AllowedValue>(options: UseStorageWithoutInitialValue<TValue>): [TValue | undefined, Setter<TValue | undefined>];
declare function useStorage<TValue extends AllowedValue>(options: UseStorageOptionsWithInitialValue<TValue>): [TValue, Setter<TValue>];

export { useStorage };
