"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/identity/farcaster/index.ts
var farcaster_exports = {};
__export(farcaster_exports, {
  useFarcasterFrameContext: () => useFarcasterFrameContext,
  useFarcasterIdentity: () => useFarcasterIdentity,
  useFarcasterMultiIdentity: () => useFarcasterMultiIdentity
});
module.exports = __toCommonJS(farcaster_exports);

// src/identity/create-frame-context-hook.ts
var import_react = require("react");

// src/identity/storage/web-storage.ts
var localStorageUnavailableMessage = "WebStorage: localStorage is not available, consider implementing your own Storage class";
var WebStorage = class {
  /**
   * Keeps latest values in memory so we don't need to read them every time
   * we set or read the value.
   */
  values;
  watchers = {};
  constructor() {
    this.values = /* @__PURE__ */ new Map();
    this.watchers = {};
  }
  get = async (key) => {
    if (this.values.has(key)) {
      return Promise.resolve(this.values.get(key));
    }
    if (typeof localStorage !== "undefined") {
      const value = localStorage.getItem(key);
      if (!value) {
        return Promise.resolve(void 0);
      }
      const parsedValue = JSON.parse(value);
      this.values.set(key, parsedValue);
      return Promise.resolve(parsedValue);
    }
    console.warn(localStorageUnavailableMessage);
    return Promise.resolve(void 0);
  };
  set = async (key, setter) => {
    const currentValue = await this.get(key);
    const newValue = setter(currentValue);
    this.values.set(key, newValue);
    this.notifyChange(key, newValue);
    if (typeof localStorage !== "undefined") {
      localStorage.setItem(key, JSON.stringify(newValue));
      return;
    }
    console.warn(localStorageUnavailableMessage);
  };
  delete = (key) => {
    this.values.delete(key);
    this.notifyChange(key, void 0);
    if (typeof localStorage !== "undefined") {
      localStorage.removeItem(key);
      return Promise.resolve();
    }
    console.warn(localStorageUnavailableMessage);
    return Promise.resolve();
  };
  watch = (key, listener) => {
    this.watchers[key] ??= [];
    this.watchers[key]?.push(listener);
    this.get(key).then((value) => {
      listener(value);
    }).catch((e) => {
      console.error(
        `@frames.js/render: Failed to get value from storage: ${e}`
      );
    });
    return () => {
      const listeners = this.watchers[key];
      if (listeners) {
        this.watchers[key] = listeners.filter(
          (watcher) => watcher !== listener
        );
      }
    };
  };
  notifyChange = (key, value) => {
    this.watchers[key]?.forEach((listener) => {
      listener(value);
    });
  };
};

// src/identity/create-frame-context-hook.ts
var defaultStorage = new WebStorage();
function createFrameContextHook(options) {
  return function useFrameContext({
    fallbackContext,
    storage = defaultStorage,
    storageKey = options.storageKey
  }) {
    const storageRef = (0, import_react.useRef)(storage);
    const [frameContext, setFrameContext] = (0, import_react.useState)(
      null
    );
    (0, import_react.useEffect)(() => {
      storageRef.current.get(storageKey).then((data) => {
        if (data) {
          setFrameContext(data);
        }
      }).catch((e) => {
        console.error(
          "@frames.js/render: Could not load frame context from storage",
          e
        );
      });
    }, [storageKey]);
    const handleSetFrameContext = (0, import_react.useCallback)(
      async (newFrameContext) => {
        await storageRef.current.set(
          storageKey,
          () => newFrameContext
        );
        setFrameContext(newFrameContext);
      },
      [storageKey]
    );
    const resetFrameContext = (0, import_react.useCallback)(async () => {
      await storageRef.current.delete(storageKey);
      setFrameContext(null);
    }, [storageKey]);
    return (0, import_react.useMemo)(
      () => ({
        frameContext: frameContext || fallbackContext,
        setFrameContext: handleSetFrameContext,
        resetFrameContext
      }),
      [fallbackContext, frameContext, handleSetFrameContext, resetFrameContext]
    );
  };
}

// src/identity/farcaster/use-farcaster-context.tsx
var useFarcasterFrameContext = createFrameContextHook({
  storageKey: "farcasterFrameContext"
});

// src/identity/farcaster/use-farcaster-identity.tsx
var import_react4 = require("react");

// src/identity/crypto.ts
var import_ed25519 = require("@noble/ed25519");
function convertKeypairToHex({
  privateKeyBytes,
  publicKeyBytes
}) {
  return {
    publicKey: `0x${Buffer.from(publicKeyBytes).toString("hex")}`,
    privateKey: `0x${Buffer.from(privateKeyBytes).toString("hex")}`
  };
}
async function createKeypairEDDSA() {
  const privateKeyBytes = import_ed25519.utils.randomPrivateKey();
  const publicKeyBytes = await (0, import_ed25519.getPublicKeyAsync)(privateKeyBytes);
  return {
    privateKeyBytes,
    publicKeyBytes
  };
}

// src/farcaster/frames.tsx
var import_core = require("@farcaster/core");
var import_viem = require("viem");
var signFrameAction = async (actionContext) => {
  const {
    frameButton,
    signer,
    buttonIndex,
    inputText,
    frameContext,
    state,
    url,
    target
  } = actionContext;
  if (!signer || signer.status === "pending_approval") {
    throw new Error("Missing signer fid to sign message");
  }
  const { message, trustedBytes } = await createFrameActionMessageWithSignerKey(
    signer.privateKey,
    {
      fid: signer.fid,
      buttonIndex,
      castId: {
        fid: frameContext.castId.fid,
        hash: (0, import_viem.hexToBytes)(frameContext.castId.hash)
      },
      state: state !== void 0 ? Buffer.from(state) : void 0,
      url: Buffer.from(url),
      // it seems the message in hubs actually requires a value here.
      inputText: inputText !== void 0 ? Buffer.from(inputText) : void 0,
      address: actionContext.type === "tx-data" || actionContext.type === "tx-post" ? (0, import_viem.hexToBytes)(actionContext.address) : void 0,
      transactionId: actionContext.type === "tx-post" ? (0, import_viem.hexToBytes)(actionContext.transactionId) : void 0
    }
  );
  if (!message) {
    throw new Error("hub error");
  }
  const searchParams = new URLSearchParams({
    postType: actionContext.type === "tx-post" ? "post" : frameButton.action,
    postUrl: target ?? ""
  });
  return {
    searchParams,
    body: {
      untrustedData: {
        fid: signer.fid,
        url,
        messageHash: `0x${Buffer.from(message.hash).toString("hex")}`,
        timestamp: message.data.timestamp,
        network: 1,
        buttonIndex: Number(message.data.frameActionBody.buttonIndex),
        castId: {
          fid: frameContext.castId.fid,
          hash: frameContext.castId.hash
        },
        inputText,
        state,
        address: actionContext.type === "tx-data" || actionContext.type === "tx-post" ? actionContext.address : void 0,
        transactionId: actionContext.type === "tx-post" ? actionContext.transactionId : void 0
      },
      trustedData: {
        messageBytes: trustedBytes
      }
    }
  };
};
async function createFrameActionMessageWithSignerKey(signerKey, {
  fid,
  url,
  buttonIndex,
  castId,
  inputText,
  state,
  address,
  transactionId
}) {
  const signer = new import_core.NobleEd25519Signer(Buffer.from(signerKey.slice(2), "hex"));
  const messageDataOptions = {
    fid,
    network: import_core.FarcasterNetwork.MAINNET
  };
  const message = await (0, import_core.makeFrameAction)(
    import_core.FrameActionBody.create({
      url,
      buttonIndex,
      castId,
      state,
      inputText: inputText !== void 0 ? Buffer.from(inputText) : void 0,
      address,
      transactionId
    }),
    messageDataOptions,
    signer
  );
  if (message.isErr()) {
    return { message: null, trustedBytes: null };
  }
  const trustedBytes = Buffer.from(
    import_core.Message.encode(message._unsafeUnwrap()).finish()
  ).toString("hex");
  return { message: message.unwrapOr(null), trustedBytes };
}

// src/farcaster/attribution.ts
var import_viem2 = require("viem");

// src/hooks/use-visibility-detection.ts
var import_react2 = require("react");
function useVisibilityDetection() {
  const listenersRef = (0, import_react2.useRef)([]);
  (0, import_react2.useEffect)(() => {
    if (typeof document !== "undefined") {
      const handleVisibilityChange = () => {
        const isVisible = !document.hidden;
        listenersRef.current.forEach((fn) => {
          fn(isVisible);
        });
      };
      document.addEventListener("visibilitychange", handleVisibilityChange);
      return () => {
        document.removeEventListener(
          "visibilitychange",
          handleVisibilityChange
        );
      };
    }
  }, []);
  return (0, import_react2.useMemo)(
    () => ({
      register(fn) {
        listenersRef.current.push(fn);
        return () => {
          listenersRef.current = listenersRef.current.filter(
            (listener) => listener !== fn
          );
        };
      }
    }),
    []
  );
}

// src/hooks/use-storage.ts
var import_react3 = require("react");
var defaultStorage2 = new WebStorage();
function useStorage({
  key,
  initialValue,
  preprocessValue,
  storage = defaultStorage2
}) {
  const storageRef = (0, import_react3.useRef)(storage);
  const preprocessValueRef = (0, import_react3.useRef)(preprocessValue);
  preprocessValueRef.current = preprocessValue;
  const initialValueRef = (0, import_react3.useRef)(initialValue);
  const [value, setValue] = (0, import_react3.useState)(
    initialValueRef.current
  );
  (0, import_react3.useEffect)(() => {
    function isValueNotUndefined(newValue) {
      return newValue !== void 0;
    }
    const listener = (newValue) => {
      if (!isValueNotUndefined(newValue)) {
        setValue(initialValueRef.current);
        return;
      }
      setValue(
        preprocessValueRef.current ? preprocessValueRef.current(newValue) : newValue
      );
    };
    const unsubscribe = storageRef.current.watch(key, listener);
    return () => {
      unsubscribe();
    };
  }, [key]);
  const setState = (0, import_react3.useCallback)(
    async (setter) => {
      await storageRef.current.set(key, (currentState) => {
        if (typeof setter !== "function") {
          return setter;
        }
        return setter(
          currentState === void 0 ? initialValueRef.current : currentState
        );
      });
    },
    [key]
  );
  return [value, setState];
}

// src/identity/farcaster/identity-poller.ts
var POLL_INTERVAL = 2e3;
var IdentityPoller = class {
  controller;
  polling = false;
  constructor() {
    this.controller = new AbortController();
  }
  isPolling = () => {
    return this.polling;
  };
  start = async (token) => {
    this.controller = new AbortController();
    this.polling = true;
    while (true) {
      if (this.controller.signal.aborted) {
        this.polling = false;
        return false;
      }
      try {
        const fcSignerRequestResponse = await fetch(
          `https://api.warpcast.com/v2/signed-key-request?token=${token}`,
          {
            method: "GET",
            headers: {
              "Content-Type": "application/json"
            }
          }
        );
        if (fcSignerRequestResponse.ok) {
          const responseBody = await fcSignerRequestResponse.json();
          if (responseBody.result.signedKeyRequest.state === "completed") {
            return responseBody.result.signedKeyRequest;
          }
        } else {
          throw new Error(
            `Warpcast API returned an error response with status code: ${fcSignerRequestResponse.status}`
          );
        }
      } catch (error) {
        console.error(
          "Error while polling for the signer approval status",
          error
        );
      }
      await new Promise((r) => {
        setTimeout(r, POLL_INTERVAL);
      });
    }
  };
  stop = () => {
    if (this.controller.signal.aborted) {
      return;
    }
    this.controller.abort();
  };
};

// src/identity/farcaster/use-farcaster-identity.tsx
var identityReducer = (state, action) => {
  switch (action.type) {
    case "IMPERSONATE": {
      const identity = {
        status: "impersonating",
        fid: action.fid,
        privateKey: action.privateKey,
        publicKey: action.publicKey,
        _id: action.id
      };
      return identity;
    }
    case "LOGOUT": {
      return { status: "init" };
    }
    case "LOGIN_START": {
      const identity = {
        _id: action.id,
        status: "pending_approval",
        privateKey: action.privateKey,
        publicKey: action.publicKey,
        token: action.token,
        deadline: action.deadline,
        signerApprovalUrl: action.signerApprovalUrl,
        requestFid: action.requestFid,
        requestSigner: action.requestSigner,
        signature: action.signature
      };
      return identity;
    }
    case "LOGIN_SUCCESS": {
      if (state.status !== "pending_approval") {
        console.warn(
          "Active identity must be selected and be in pending_approval status to be approved"
        );
        return state;
      }
      const updatedIdentity = {
        ...state,
        status: "approved",
        fid: action.signedKeyRequest.userFid,
        signedKeyRequest: action.signedKeyRequest
      };
      return updatedIdentity;
    }
    default:
      return state;
  }
};
var defaultStorage3 = new WebStorage();
var defaultGenerateUserId = () => {
  return Date.now();
};
function useFarcasterIdentity({
  onMissingIdentity,
  enableIdentityPolling = true,
  signerUrl = "/signer",
  storage = defaultStorage3,
  storageKey = "farcasterIdentity",
  visibilityChangeDetectionHook = useVisibilityDetection,
  onImpersonate,
  onLogIn,
  onLogInStart,
  onLogOut,
  generateUserId = defaultGenerateUserId
}) {
  const storageRef = (0, import_react4.useRef)(storage);
  const identityPoller = (0, import_react4.useRef)(new IdentityPoller()).current;
  const [isLoading, setIsLoading] = (0, import_react4.useState)(false);
  const [state, setState] = useStorage({
    key: storageKey,
    storage: storageRef.current,
    initialValue: { status: "init" },
    preprocessValue(value) {
      if (value.status === "pending_approval" && value.deadline < Math.floor(Date.now() / 1e3)) {
        return { status: "init" };
      }
      return value;
    }
  });
  const onImpersonateRef = (0, import_react4.useRef)(onImpersonate);
  onImpersonateRef.current = onImpersonate;
  const onLogInRef = (0, import_react4.useRef)(onLogIn);
  onLogInRef.current = onLogIn;
  const onLogInStartRef = (0, import_react4.useRef)(onLogInStart);
  onLogInStartRef.current = onLogInStart;
  const onLogOutRef = (0, import_react4.useRef)(onLogOut);
  onLogOutRef.current = onLogOut;
  const generateUserIdRef = (0, import_react4.useRef)(generateUserId);
  generateUserIdRef.current = generateUserId;
  const createFarcasterSigner = (0, import_react4.useCallback)(async () => {
    try {
      const keypair = await createKeypairEDDSA();
      const keypairString = convertKeypairToHex(keypair);
      const authorizationResponse = await fetch(
        // real signer or local one are handled by local route so we don't need to expose anything to client side bundle
        signerUrl,
        {
          method: "POST",
          body: JSON.stringify({
            publicKey: keypairString.publicKey
          })
        }
      );
      const authorizationBody = await authorizationResponse.json();
      if (authorizationResponse.status === 200) {
        const { signature, requestFid, deadline, requestSigner } = authorizationBody;
        const {
          result: { signedKeyRequest }
        } = await (await fetch(`https://api.warpcast.com/v2/signed-key-requests`, {
          method: "POST",
          headers: {
            "Content-Type": "application/json"
          },
          body: JSON.stringify({
            key: keypairString.publicKey,
            signature,
            requestFid,
            deadline
          })
        })).json();
        const deepLinkUrl = new URL(signedKeyRequest.deeplinkUrl);
        const signedKeyRequestToken = deepLinkUrl.searchParams.get("token");
        const signerApprovalUrl = new URL(
          "https://client.warpcast.com/deeplinks/signed-key-request"
        );
        if (!signedKeyRequestToken) {
          throw new Error("No token found in the deep link URL");
        }
        signerApprovalUrl.searchParams.set("token", signedKeyRequestToken);
        await setState((currentState) => {
          const newState = identityReducer(currentState, {
            type: "LOGIN_START",
            id: generateUserIdRef.current(),
            publicKey: keypairString.publicKey,
            privateKey: keypairString.privateKey,
            deadline,
            token: signedKeyRequestToken,
            signerApprovalUrl: signerApprovalUrl.toString(),
            requestFid: parseInt(requestFid, 10),
            requestSigner,
            signature
          });
          if (newState.status === "pending_approval") {
            onLogInStartRef.current?.(newState);
          }
          return newState;
        });
        return {
          token: signedKeyRequestToken,
          signerApprovalUrl: signerApprovalUrl.toString()
        };
      } else if ("message" in authorizationBody) {
        throw new Error(authorizationBody.message);
      }
      throw new Error("Could not request signer approval");
    } catch (error) {
      console.error("@frames.js/render: API Call failed", error);
      throw error;
    }
  }, [setState, signerUrl]);
  const impersonateUser = (0, import_react4.useCallback)(
    async (fid) => {
      try {
        setIsLoading(true);
        const keypair = await createKeypairEDDSA();
        const { privateKey, publicKey } = convertKeypairToHex(keypair);
        await setState((currentState) => {
          const newState = identityReducer(currentState, {
            type: "IMPERSONATE",
            id: generateUserIdRef.current(),
            fid,
            privateKey,
            publicKey
          });
          if (newState.status === "impersonating") {
            onImpersonateRef.current?.(newState);
          }
          return newState;
        });
      } finally {
        setIsLoading(false);
      }
    },
    [setState]
  );
  const onSignerlessFramePress = (0, import_react4.useCallback)(() => {
    onMissingIdentity();
    return Promise.resolve();
  }, [onMissingIdentity]);
  const createSigner = (0, import_react4.useCallback)(async () => {
    setIsLoading(true);
    const result = await createFarcasterSigner();
    setIsLoading(false);
    return result;
  }, [createFarcasterSigner]);
  const logout = (0, import_react4.useCallback)(async () => {
    await setState((currentState) => {
      if (currentState.status !== "init") {
        onLogOutRef.current?.(currentState);
      }
      return identityReducer(currentState, { type: "LOGOUT" });
    });
  }, [setState]);
  const farcasterUser = state.status === "init" ? null : state;
  const visibilityDetector = visibilityChangeDetectionHook();
  (0, import_react4.useEffect)(() => {
    if (farcasterUser && farcasterUser.status === "pending_approval" && enableIdentityPolling) {
      const startPolling = () => {
        identityPoller.start(farcasterUser.token).then((signedKeyRequest) => {
          if (signedKeyRequest) {
            return setState((currentState) => {
              const newState = identityReducer(currentState, {
                type: "LOGIN_SUCCESS",
                signedKeyRequest
              });
              if (newState.status === "approved") {
                onLogInRef.current?.(newState);
              }
              return newState;
            });
          }
        }).catch((e) => {
          console.error(
            "Error while polling for the signer approval status",
            e
          );
        });
      };
      const unregisterVisibilityChangeListener = visibilityDetector.register(
        (visible) => {
          if (visible) {
            startPolling();
          } else {
            identityPoller.stop();
          }
        }
      );
      startPolling();
      return () => {
        identityPoller.stop();
        unregisterVisibilityChangeListener();
      };
    }
  }, [
    farcasterUser,
    identityPoller,
    visibilityDetector,
    setState,
    enableIdentityPolling
  ]);
  return (0, import_react4.useMemo)(
    () => ({
      signer: farcasterUser,
      hasSigner: farcasterUser?.status === "approved" || farcasterUser?.status === "impersonating",
      signFrameAction,
      isLoadingSigner: isLoading,
      impersonateUser,
      onSignerlessFramePress,
      createSigner,
      logout,
      identityPoller
    }),
    [
      farcasterUser,
      identityPoller,
      impersonateUser,
      isLoading,
      logout,
      createSigner,
      onSignerlessFramePress
    ]
  );
}

// src/identity/farcaster/use-farcaster-multi-identity.tsx
var import_react5 = require("react");
var identityReducer2 = (state, action) => {
  switch (action.type) {
    case "IMPERSONATE": {
      const identity = {
        status: "impersonating",
        fid: action.fid,
        privateKey: action.privateKey,
        publicKey: action.publicKey,
        _id: action.id
      };
      return {
        activeIdentity: identity,
        identities: state.identities.concat(identity)
      };
    }
    case "SELECT_IDENTITY": {
      const activeIdentity = state.identities.find((identity) => identity._id === action.id) ?? null;
      return {
        ...state,
        activeIdentity
      };
    }
    case "LOGOUT": {
      return {
        activeIdentity: null,
        identities: state.identities
      };
    }
    case "REMOVE": {
      const { activeIdentity } = state;
      if (!activeIdentity) {
        return state;
      }
      return {
        activeIdentity: null,
        identities: state.identities.filter(
          (identity) => identity._id !== activeIdentity._id
        )
      };
    }
    case "LOGIN_START": {
      const identity = {
        _id: action.id,
        status: "pending_approval",
        privateKey: action.privateKey,
        publicKey: action.publicKey,
        token: action.token,
        deadline: action.deadline,
        signerApprovalUrl: action.signerApprovalUrl,
        requestFid: action.requestFid,
        requestSigner: action.requestSigner,
        signature: action.signature
      };
      const identities = [
        ...state.identities.filter(
          (storedIdentity) => storedIdentity.status !== "pending_approval"
        ),
        // Add new pending approval
        identity
      ];
      return {
        activeIdentity: identity,
        identities
      };
    }
    case "LOGIN_SUCCESS": {
      const { activeIdentity } = state;
      if (activeIdentity?.status !== "pending_approval") {
        console.warn(
          "Active identity must be selected and be in pending_approval status to be approved"
        );
        return state;
      }
      const updatedActiveIdentity = {
        ...activeIdentity,
        status: "approved",
        fid: action.signedKeyRequest.userFid,
        signedKeyRequest: action.signedKeyRequest
      };
      return {
        activeIdentity: updatedActiveIdentity,
        identities: state.identities.map((identity) => {
          if (identity._id === activeIdentity._id) {
            return updatedActiveIdentity;
          }
          return identity;
        })
      };
    }
    default:
      return state;
  }
};
var defaultStorage4 = new WebStorage();
var defaultGenerateUserId2 = () => Date.now();
function useFarcasterMultiIdentity({
  onMissingIdentity,
  signerUrl = "/signer",
  storage = defaultStorage4,
  storageKey = "farcasterIdentities",
  visibilityChangeDetectionHook = useVisibilityDetection,
  onImpersonate,
  onLogIn,
  onLogInStart,
  onLogOut,
  onIdentityRemove,
  onIdentitySelect,
  generateUserId = defaultGenerateUserId2
}) {
  const storageRef = (0, import_react5.useRef)(storage);
  const identityPoller = (0, import_react5.useRef)(new IdentityPoller()).current;
  const [isLoading, setIsLoading] = (0, import_react5.useState)(false);
  const [state, setState] = useStorage({
    key: storageKey,
    storage: storageRef.current,
    preprocessValue(value) {
      const stateWithoutExpiredPendingApprovals = {
        activeIdentity: value.activeIdentity,
        identities: value.identities.filter(
          (signer) => signer.status !== "pending_approval" || signer.deadline && signer.deadline > Math.floor(Date.now() / 1e3)
        )
      };
      stateWithoutExpiredPendingApprovals.activeIdentity = stateWithoutExpiredPendingApprovals.identities.find(
        (identity) => identity._id === stateWithoutExpiredPendingApprovals.activeIdentity?._id
      ) ?? null;
      return stateWithoutExpiredPendingApprovals;
    },
    initialValue: {
      activeIdentity: null,
      identities: []
    }
  });
  const onImpersonateRef = (0, import_react5.useRef)(onImpersonate);
  onImpersonateRef.current = onImpersonate;
  const onLogInRef = (0, import_react5.useRef)(onLogIn);
  onLogInRef.current = onLogIn;
  const onLogInStartRef = (0, import_react5.useRef)(onLogInStart);
  onLogInStartRef.current = onLogInStart;
  const onLogOutRef = (0, import_react5.useRef)(onLogOut);
  onLogOutRef.current = onLogOut;
  const onIdentityRemoveRef = (0, import_react5.useRef)(onIdentityRemove);
  onIdentityRemoveRef.current = onIdentityRemove;
  const onIdentitySelectRef = (0, import_react5.useRef)(onIdentitySelect);
  onIdentitySelectRef.current = onIdentitySelect;
  const generateUserIdRef = (0, import_react5.useRef)(generateUserId);
  generateUserIdRef.current = generateUserId;
  const createFarcasterSigner = (0, import_react5.useCallback)(async () => {
    try {
      const keypair = await createKeypairEDDSA();
      const keypairString = convertKeypairToHex(keypair);
      const authorizationResponse = await fetch(
        // real signer or local one are handled by local route so we don't need to expose anything to client side bundle
        signerUrl,
        {
          method: "POST",
          body: JSON.stringify({
            publicKey: keypairString.publicKey
          })
        }
      );
      const authorizationBody = await authorizationResponse.json();
      if (authorizationResponse.status === 200) {
        const {
          signature,
          requestFid,
          deadline,
          requestSigner,
          sponsorship
        } = authorizationBody;
        const {
          result: { signedKeyRequest }
        } = await (await fetch(`https://api.warpcast.com/v2/signed-key-requests`, {
          method: "POST",
          headers: {
            "Content-Type": "application/json"
          },
          body: JSON.stringify({
            key: keypairString.publicKey,
            signature,
            requestFid,
            deadline,
            sponsorship
          })
        })).json();
        const deepLinkUrl = new URL(signedKeyRequest.deeplinkUrl);
        const signedKeyRequestToken = deepLinkUrl.searchParams.get("token");
        const signerApprovalUrl = new URL(
          "https://client.warpcast.com/deeplinks/signed-key-request"
        );
        if (!signedKeyRequestToken) {
          throw new Error("No token found in the deep link URL");
        }
        signerApprovalUrl.searchParams.set("token", signedKeyRequestToken);
        await setState((currentState) => {
          const newState = identityReducer2(currentState, {
            type: "LOGIN_START",
            id: generateUserIdRef.current(),
            publicKey: keypairString.publicKey,
            privateKey: keypairString.privateKey,
            deadline,
            token: signedKeyRequestToken,
            signerApprovalUrl: signerApprovalUrl.toString(),
            requestFid: parseInt(requestFid, 10),
            requestSigner,
            signature
          });
          if (newState.activeIdentity?.status === "pending_approval") {
            onLogInStartRef.current?.(newState.activeIdentity);
          }
          return newState;
        });
        return {
          token: signedKeyRequestToken,
          signerApprovalUrl: signerApprovalUrl.toString()
        };
      } else if ("message" in authorizationBody) {
        throw new Error(authorizationBody.message);
      }
      throw new Error("Could not request signer approval.");
    } catch (error) {
      console.error("@frames.js/render: API Call failed", error);
      throw error;
    }
  }, [setState, signerUrl]);
  const impersonateUser = (0, import_react5.useCallback)(
    async (fid) => {
      try {
        setIsLoading(true);
        const keypair = await createKeypairEDDSA();
        const { privateKey, publicKey } = convertKeypairToHex(keypair);
        await setState((currentState) => {
          const newState = identityReducer2(currentState, {
            type: "IMPERSONATE",
            id: generateUserIdRef.current(),
            fid,
            privateKey,
            publicKey
          });
          if (newState.activeIdentity?.status === "impersonating") {
            onImpersonateRef.current?.(newState.activeIdentity);
          }
          return newState;
        });
      } finally {
        setIsLoading(false);
      }
    },
    [setState]
  );
  const onSignerlessFramePress = (0, import_react5.useCallback)(() => {
    onMissingIdentity();
    return Promise.resolve();
  }, [onMissingIdentity]);
  const createSigner = (0, import_react5.useCallback)(async () => {
    setIsLoading(true);
    const result = await createFarcasterSigner();
    setIsLoading(false);
    return result;
  }, [createFarcasterSigner]);
  const logout = (0, import_react5.useCallback)(async () => {
    await setState((currentState) => {
      if (currentState.activeIdentity) {
        onLogOutRef.current?.(currentState.activeIdentity);
      }
      return identityReducer2(currentState, { type: "LOGOUT" });
    });
  }, [setState]);
  const removeIdentity = (0, import_react5.useCallback)(async () => {
    await setState((currentState) => {
      if (currentState.activeIdentity) {
        onIdentityRemoveRef.current?.(currentState.activeIdentity);
      }
      return identityReducer2(currentState, { type: "REMOVE" });
    });
  }, [setState]);
  const farcasterUser = state.activeIdentity;
  const visibilityDetector = visibilityChangeDetectionHook();
  (0, import_react5.useEffect)(() => {
    if (farcasterUser && farcasterUser.status === "pending_approval") {
      const startPolling = () => {
        identityPoller.start(farcasterUser.token).then((signedKeyRequest) => {
          if (signedKeyRequest) {
            return setState((currentState) => {
              const newState = identityReducer2(currentState, {
                type: "LOGIN_SUCCESS",
                signedKeyRequest
              });
              if (newState.activeIdentity?.status === "approved") {
                onLogInRef.current?.(newState.activeIdentity);
              }
              return newState;
            });
          }
        }).catch((e) => {
          console.error(
            "Error while polling for the signer approval status",
            e
          );
        });
      };
      const unregisterVisibilityChangeListener = visibilityDetector.register(
        (visible) => {
          if (visible) {
            startPolling();
          } else {
            identityPoller.stop();
          }
        }
      );
      startPolling();
      return () => {
        identityPoller.stop();
        unregisterVisibilityChangeListener();
      };
    }
  }, [farcasterUser, identityPoller, visibilityDetector, setState]);
  const selectIdentity = (0, import_react5.useCallback)(
    async (id) => {
      await setState((currentState) => {
        const newState = identityReducer2(currentState, {
          type: "SELECT_IDENTITY",
          id
        });
        if (newState.activeIdentity) {
          onIdentitySelectRef.current?.(newState.activeIdentity);
        }
        return newState;
      });
    },
    [setState]
  );
  return (0, import_react5.useMemo)(
    () => ({
      signer: farcasterUser,
      hasSigner: farcasterUser?.status === "approved" || farcasterUser?.status === "impersonating",
      signFrameAction,
      isLoadingSigner: isLoading,
      impersonateUser,
      onSignerlessFramePress,
      createSigner,
      logout,
      removeIdentity,
      identities: state.identities,
      selectIdentity,
      identityPoller
    }),
    [
      farcasterUser,
      identityPoller,
      impersonateUser,
      isLoading,
      logout,
      createSigner,
      onSignerlessFramePress,
      removeIdentity,
      selectIdentity,
      state.identities
    ]
  );
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useFarcasterFrameContext,
  useFarcasterIdentity,
  useFarcasterMultiIdentity
});
