import { a as FarcasterSignerState } from '../../types-0pyvZInF.cjs';
import { Storage } from '../types.cjs';
import { useVisibilityDetection } from '../../hooks/use-visibility-detection.cjs';
import { IdentityPoller } from './identity-poller.cjs';
import { FarcasterSigner, FarcasterCreateSignerResult, FarcasterSignerImpersonating, FarcasterSignerApproved, FarcasterSignerPendingApproval } from './types.cjs';
import 'frames.js';
import 'react';
import 'frames.js/frame-parsers';
import 'frames.js/types';
import '@farcaster/core';

type UseFarcasterMultiIdentityOptions = {
    /**
     * Called when it is required to create a new signer in order to proceed
     */
    onMissingIdentity: () => void;
    /**
     * URL to signer endpoint
     *
     * @defaultValue '/signer'
     */
    signerUrl?: string;
    /**
     * @defaultValue WebStorage
     */
    storage?: Storage;
    /**
     * @defaultValue 'farcasterIdentities'
     */
    storageKey?: string;
    /**
     * Used to detect if the current context is visible, this affects the polling of the signer approval status.
     */
    visibilityChangeDetectionHook?: typeof useVisibilityDetection;
    onImpersonate?: (identity: FarcasterSignerImpersonating) => void;
    onLogIn?: (identity: FarcasterSignerApproved) => void;
    onLogInStart?: (identity: FarcasterSignerPendingApproval) => void;
    onLogOut?: (identity: FarcasterSigner) => void;
    onIdentityRemove?: (identity: FarcasterSigner) => void;
    onIdentitySelect?: (identity: FarcasterSigner) => void;
    generateUserId?: () => string | number;
};
type FarcasterMultiSignerInstance = FarcasterSignerState<FarcasterSigner | null> & {
    createSigner: () => Promise<FarcasterCreateSignerResult>;
    impersonateUser: (fid: number) => Promise<void>;
    removeIdentity: () => Promise<void>;
    identities: FarcasterSigner[];
    selectIdentity: (id: number | string) => Promise<void>;
    identityPoller: IdentityPoller;
};
declare function useFarcasterMultiIdentity({ onMissingIdentity, signerUrl, storage, storageKey, visibilityChangeDetectionHook, onImpersonate, onLogIn, onLogInStart, onLogOut, onIdentityRemove, onIdentitySelect, generateUserId, }: UseFarcasterMultiIdentityOptions): FarcasterMultiSignerInstance;

export { type FarcasterMultiSignerInstance, useFarcasterMultiIdentity };
