"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/identity/lens/use-lens-identity.tsx
var use_lens_identity_exports = {};
__export(use_lens_identity_exports, {
  useLensIdentity: () => useLensIdentity
});
module.exports = __toCommonJS(use_lens_identity_exports);
var import_rainbowkit = require("@rainbow-me/rainbowkit");
var import_react = require("react");
var import_wagmi = require("wagmi");
var import_actions = require("wagmi/actions");
var import_client = require("@lens-protocol/client");

// src/identity/storage/web-storage.ts
var localStorageUnavailableMessage = "WebStorage: localStorage is not available, consider implementing your own Storage class";
var WebStorage = class {
  /**
   * Keeps latest values in memory so we don't need to read them every time
   * we set or read the value.
   */
  values;
  watchers = {};
  constructor() {
    this.values = /* @__PURE__ */ new Map();
    this.watchers = {};
  }
  get = async (key) => {
    if (this.values.has(key)) {
      return Promise.resolve(this.values.get(key));
    }
    if (typeof localStorage !== "undefined") {
      const value = localStorage.getItem(key);
      if (!value) {
        return Promise.resolve(void 0);
      }
      const parsedValue = JSON.parse(value);
      this.values.set(key, parsedValue);
      return Promise.resolve(parsedValue);
    }
    console.warn(localStorageUnavailableMessage);
    return Promise.resolve(void 0);
  };
  set = async (key, setter) => {
    const currentValue = await this.get(key);
    const newValue = setter(currentValue);
    this.values.set(key, newValue);
    this.notifyChange(key, newValue);
    if (typeof localStorage !== "undefined") {
      localStorage.setItem(key, JSON.stringify(newValue));
      return;
    }
    console.warn(localStorageUnavailableMessage);
  };
  delete = (key) => {
    this.values.delete(key);
    this.notifyChange(key, void 0);
    if (typeof localStorage !== "undefined") {
      localStorage.removeItem(key);
      return Promise.resolve();
    }
    console.warn(localStorageUnavailableMessage);
    return Promise.resolve();
  };
  watch = (key, listener) => {
    this.watchers[key] ??= [];
    this.watchers[key]?.push(listener);
    this.get(key).then((value) => {
      listener(value);
    }).catch((e) => {
      console.error(
        `@frames.js/render: Failed to get value from storage: ${e}`
      );
    });
    return () => {
      const listeners = this.watchers[key];
      if (listeners) {
        this.watchers[key] = listeners.filter(
          (watcher) => watcher !== listener
        );
      }
    };
  };
  notifyChange = (key, value) => {
    this.watchers[key]?.forEach((listener) => {
      listener(value);
    });
  };
};

// src/identity/lens/use-lens-identity.tsx
var defaultStorage = new WebStorage();
function useLensIdentity({
  storage = defaultStorage,
  storageKey = "lensProfile"
} = {}) {
  const storageRef = (0, import_react.useRef)(storage);
  const [isLoading, setIsLoading] = (0, import_react.useState)(false);
  const [lensSigner, setLensSigner] = (0, import_react.useState)(null);
  const [showProfileSelector, setShowProfileSelector] = (0, import_react.useState)(false);
  const [availableProfiles, setAvailableProfiles] = (0, import_react.useState)([]);
  const connect = (0, import_rainbowkit.useConnectModal)();
  const config = (0, import_wagmi.useConfig)();
  const { address } = (0, import_wagmi.useAccount)();
  const activeConnection = (0, import_wagmi.useConnections)();
  const lensClient = (0, import_react.useRef)(
    new import_client.LensClient({
      environment: import_client.production
    })
  ).current;
  (0, import_react.useEffect)(() => {
    storageRef.current.get(storageKey).then((storedData) => {
      if (storedData) {
        setLensSigner(storedData);
      }
    }).catch((e) => {
      console.error("@frames.js/render: Could not get the Lens profile", e);
    });
  }, [storageKey]);
  const logout = (0, import_react.useCallback)(async () => {
    await storageRef.current.delete(storageKey);
    setLensSigner(null);
  }, [storageKey]);
  const handleSelectProfile = (0, import_react.useCallback)(
    async (profile) => {
      try {
        if (!address) {
          throw new Error("No wallet connected");
        }
        setShowProfileSelector(false);
        const { id, text } = await lensClient.authentication.generateChallenge({
          signedBy: address,
          for: profile.id
        });
        const signature = await (0, import_actions.signMessage)(config, {
          message: {
            raw: typeof text === "string" ? Buffer.from(text) : Buffer.from(text)
          }
        });
        await lensClient.authentication.authenticate({ id, signature });
        const accessTokenResult = await lensClient.authentication.getAccessToken();
        const identityTokenResult = await lensClient.authentication.getIdentityToken();
        const accessToken = accessTokenResult.unwrap();
        const identityToken = identityTokenResult.unwrap();
        const profileId = await lensClient.authentication.getProfileId();
        const profileInfo = await lensClient.profile.fetch({
          forProfileId: profileId
        });
        const handle = profileInfo?.handle?.localName ? `${profileInfo.handle.localName}.lens` : "";
        if (profileId) {
          const signer = {
            accessToken,
            profileId,
            address,
            identityToken,
            handle
          };
          await storageRef.current.set(storageKey, () => signer);
          setLensSigner(signer);
        }
      } catch (error) {
        console.error("@frames.js/render: Create Lens signer failed", error);
      }
    },
    [address, config, lensClient.authentication, lensClient.profile, storageKey]
  );
  const onSignerlessFramePress = (0, import_react.useCallback)(async () => {
    try {
      setIsLoading(true);
      if (!lensSigner) {
        if (!address) {
          connect.openConnectModal?.();
          return;
        }
        const managedProfiles = await lensClient.wallet.profilesManaged({
          for: address
        });
        const profiles = managedProfiles.items.map((p) => ({
          id: p.id,
          handle: p.handle ? `${p.handle.localName}.lens` : void 0
        }));
        if (!profiles[0]) {
          throw new Error("No Lens profiles managed by connected address");
        }
        if (managedProfiles.items.length > 1) {
          setAvailableProfiles(profiles);
          setShowProfileSelector(true);
        } else {
          await handleSelectProfile(profiles[0]);
        }
      }
    } catch (error) {
      console.error("@frames.js/render: Create Lens signer failed", error);
    } finally {
      setIsLoading(false);
    }
  }, [address, connect, handleSelectProfile, lensClient.wallet, lensSigner]);
  const signFrameAction = (0, import_react.useCallback)(
    async (actionContext) => {
      if (!lensSigner) {
        throw new Error("No lens signer active");
      }
      const profileManagers = await lensClient.profile.managers({
        for: lensSigner.profileId
      });
      const lensManagerEnabled = profileManagers.items.some(
        (manager) => manager.isLensManager
      );
      if (lensManagerEnabled) {
        const result = await lensClient.frames.signFrameAction({
          url: actionContext.url,
          inputText: actionContext.inputText || "",
          state: actionContext.state || "",
          buttonIndex: actionContext.buttonIndex,
          actionResponse: actionContext.type === "tx-post" ? actionContext.transactionId : "",
          profileId: lensSigner.profileId,
          pubId: actionContext.frameContext.pubId || "",
          specVersion: "1.0.0"
        });
        if (result.isFailure()) {
          throw new Error("credential expired or not authenticated");
        }
        const searchParams2 = new URLSearchParams({
          postType: actionContext.type === "tx-post" ? "post" : actionContext.frameButton.action,
          postUrl: actionContext.frameButton.target ?? actionContext.target ?? ""
        });
        return {
          body: {
            clientProtocol: "lens@1.0.0",
            untrustedData: {
              ...result.value.signedTypedData.value,
              identityToken: lensSigner.identityToken,
              unixTimestamp: Date.now()
            },
            trustedData: {
              messageBytes: result.value.signature
            }
          },
          searchParams: searchParams2
        };
      }
      const typedData = await lensClient.frames.createFrameTypedData({
        url: actionContext.url,
        inputText: actionContext.inputText || "",
        state: actionContext.state || "",
        buttonIndex: actionContext.buttonIndex,
        actionResponse: actionContext.type === "tx-post" ? actionContext.transactionId : "",
        profileId: lensSigner.profileId,
        pubId: actionContext.frameContext.pubId || "",
        specVersion: "1.0.0",
        deadline: Math.floor(Date.now() / 1e3) + 86400
        // 1 day
      });
      if (activeConnection[0]?.chainId !== typedData.domain.chainId) {
        await (0, import_actions.switchChain)(config, { chainId: typedData.domain.chainId });
      }
      const signature = await (0, import_actions.signTypedData)(config, {
        domain: {
          ...typedData.domain,
          verifyingContract: typedData.domain.verifyingContract
        },
        types: typedData.types,
        message: typedData.value,
        primaryType: "FrameData"
      });
      const searchParams = new URLSearchParams({
        postType: actionContext.type === "tx-post" ? "post" : actionContext.frameButton.action,
        postUrl: actionContext.target ?? ""
      });
      return {
        body: {
          clientProtocol: "lens@1.0.0",
          untrustedData: {
            ...typedData.value,
            identityToken: lensSigner.identityToken,
            unixTimestamp: Date.now()
          },
          trustedData: {
            messageBytes: signature
          }
        },
        searchParams
      };
    },
    [
      activeConnection,
      config,
      lensClient.frames,
      lensClient.profile,
      lensSigner
    ]
  );
  const closeProfileSelector = (0, import_react.useCallback)(() => {
    setShowProfileSelector(false);
  }, []);
  return (0, import_react.useMemo)(
    () => ({
      signer: lensSigner,
      hasSigner: !!lensSigner?.accessToken,
      signFrameAction,
      isLoadingSigner: isLoading,
      onSignerlessFramePress,
      logout,
      showProfileSelector,
      closeProfileSelector,
      availableProfiles,
      handleSelectProfile
    }),
    [
      availableProfiles,
      closeProfileSelector,
      handleSelectProfile,
      isLoading,
      lensSigner,
      logout,
      onSignerlessFramePress,
      showProfileSelector,
      signFrameAction
    ]
  );
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useLensIdentity
});
