import { LensFrameContext } from './use-lens-context.cjs';
import { z as SignerStateInstance } from '../../types-0pyvZInF.cjs';
import { Storage } from '../types.cjs';
import '../../hooks/use-visibility-detection.cjs';
import 'frames.js';
import 'react';
import 'frames.js/frame-parsers';
import 'frames.js/types';
import '@farcaster/core';

type LensProfile = {
    id: string;
    handle?: string;
};
type LensSigner = {
    profileId: string;
    accessToken: string;
    identityToken: string;
    handle: string;
    address: `0x${string}`;
};
type LensFrameRequest = {
    clientProtocol: string;
    untrustedData: {
        specVersion: string;
        profileId: string;
        pubId: string;
        url: string;
        buttonIndex: number;
        unixTimestamp: number;
        deadline?: number;
        inputText?: string;
        state?: string;
        actionResponse?: string;
        identityToken: string;
    };
    trustedData: {
        messageBytes: string;
    };
};
type LensSignerInstance = SignerStateInstance<LensSigner, LensFrameRequest, LensFrameContext> & {
    showProfileSelector: boolean;
    availableProfiles: LensProfile[];
    handleSelectProfile: (profile: LensProfile) => Promise<void>;
    closeProfileSelector: () => void;
};
type LensIdentityOptions = {
    /**
     * @defaultValue WebStorage
     */
    storage?: Storage;
    /**
     * @defaultValue "lensProfile"
     */
    storageKey?: string;
};
declare function useLensIdentity({ storage, storageKey, }?: LensIdentityOptions): LensSignerInstance;

export { type LensProfile, type LensSigner, useLensIdentity };
