import { createElement } from 'react';
import { Y as FrameState } from '../types-0pyvZInF.cjs';
import { FrameUIComponents as FrameUIComponents$1, FrameUIComponentStylingProps, FrameMessage } from './types.cjs';
import 'frames.js';
import 'frames.js/frame-parsers';
import 'frames.js/types';
import '@farcaster/core';

type FrameUIComponents<TStylingProps extends Record<string, unknown>> = Partial<FrameUIComponents$1<TStylingProps>>;
type FrameUITheme<TStylingProps extends Record<string, unknown>> = Partial<FrameUIComponentStylingProps<TStylingProps>>;
type BaseFrameUIProps<TStylingProps extends Record<string, unknown>> = {
    frameState: FrameState<any, any>;
    /**
     * Renders also frames that contain only image and at least one button
     *
     * @defaultValue false
     */
    allowPartialFrame?: boolean;
    /**
     * Renders debug image if available
     */
    enableImageDebugging?: boolean;
    components: Required<FrameUIComponents<TStylingProps>>;
    theme?: FrameUITheme<TStylingProps> | undefined;
    /**
     * Called when an error occurs in response to frame button press
     *
     * @defaultValue console.error()
     */
    onError?: (error: Error) => void;
    /**
     * Called when a messages is returned in response to frame button press
     */
    onMessage?: (message: FrameMessage) => void;
    /**
     * Custom createElement function to use when rendering components.
     *
     * This is useful for libraries like Nativewind that require a custom createElement function.
     *
     * @defaultValue React.createElement
     */
    createElement?: typeof createElement;
};
declare function BaseFrameUI<TStylingProps extends Record<string, unknown>>({ frameState, components, theme, allowPartialFrame, enableImageDebugging, onError, onMessage, createElement, }: BaseFrameUIProps<TStylingProps>): JSX.Element | null;

export { BaseFrameUI, type BaseFrameUIProps, type FrameUIComponents, type FrameUITheme };
