"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/ui/index.tsx
var ui_exports = {};
__export(ui_exports, {
  FrameUI: () => FrameUI
});
module.exports = __toCommonJS(ui_exports);
var import_react2 = __toESM(require("react"), 1);

// src/ui/frame.base.tsx
var import_react = require("react");

// src/ui/utils.ts
function isPartialFrameStackItem(stackItem) {
  return stackItem.status === "done" && stackItem.frameResult.status === "failure" && !!stackItem.frameResult.frame.image && !!stackItem.frameResult.frame.buttons && stackItem.frameResult.frame.buttons.length > 0;
}
function getErrorMessageFromFramesStackItem(item) {
  if (item.status === "message") {
    return item.message;
  }
  if (item.requestError instanceof Error) {
    return item.requestError.message;
  }
  return "An error occurred";
}

// src/ui/frame.base.tsx
function defaultMessageHandler() {
}
function BaseFrameUI({
  frameState,
  components,
  theme,
  allowPartialFrame = false,
  enableImageDebugging = false,
  // eslint-disable-next-line no-console -- provide at least some feedback to the user
  onError = console.error,
  onMessage = defaultMessageHandler,
  createElement = import_react.createElement
}) {
  const [isImageLoading, setIsImageLoading] = (0, import_react.useState)(true);
  const { currentFrameStackItem } = frameState;
  const rootRef = (0, import_react.useRef)(null);
  const rootDimensionsRef = (0, import_react.useRef)();
  const onImageLoadEnd = (0, import_react.useCallback)(() => {
    setIsImageLoading(false);
  }, []);
  (0, import_react.useEffect)(() => {
    if (currentFrameStackItem?.status === "pending") {
      setIsImageLoading(true);
    }
  }, [currentFrameStackItem?.status]);
  const onErrorRef = (0, import_react.useRef)(onError);
  onErrorRef.current = onError;
  (0, import_react.useEffect)(() => {
    if (currentFrameStackItem?.status === "requestError") {
      onErrorRef.current(currentFrameStackItem.requestError);
    }
  }, [currentFrameStackItem]);
  const onMessageRef = (0, import_react.useRef)(onMessage);
  onMessageRef.current = onMessage;
  (0, import_react.useEffect)(() => {
    if (currentFrameStackItem?.status === "message") {
      onMessageRef.current({
        message: currentFrameStackItem.message,
        status: currentFrameStackItem.type === "info" ? "message" : "error"
      });
    }
  }, [currentFrameStackItem]);
  if (!frameState.homeframeUrl) {
    return components.Error(
      { message: "Missing frame url" },
      theme?.Error || {}
    );
  }
  if (!currentFrameStackItem) {
    return null;
  }
  let frameUiState;
  const previousFrame = frameState.framesStack[frameState.framesStack.length - 1]?.frameResult?.frame;
  switch (currentFrameStackItem.status) {
    case "requestError": {
      if ("sourceFrame" in currentFrameStackItem.request && currentFrameStackItem.request.sourceFrame) {
        frameUiState = {
          status: "complete",
          frame: currentFrameStackItem.request.sourceFrame,
          isImageLoading,
          id: currentFrameStackItem.timestamp.getTime(),
          frameState
        };
      } else {
        return components.Error(
          {
            message: "Failed to load frame",
            error: currentFrameStackItem.requestError
          },
          theme?.Error || {}
        );
      }
      break;
    }
    case "message":
      if (!currentFrameStackItem.request.sourceFrame) {
        return components.Message(
          {
            message: currentFrameStackItem.message,
            status: currentFrameStackItem.type === "error" ? "error" : "message"
          },
          theme?.Message ?? {}
        );
      }
      frameUiState = {
        status: "complete",
        frame: currentFrameStackItem.request.sourceFrame,
        isImageLoading,
        id: currentFrameStackItem.timestamp.getTime(),
        frameState
      };
      break;
    case "doneRedirect": {
      if (!currentFrameStackItem.request.sourceFrame) {
        frameUiState = {
          status: "loading",
          id: currentFrameStackItem.timestamp.getTime(),
          frameState
        };
      } else {
        frameUiState = {
          status: "complete",
          frame: currentFrameStackItem.request.sourceFrame,
          isImageLoading,
          id: currentFrameStackItem.timestamp.getTime(),
          frameState
        };
      }
      break;
    }
    case "done": {
      if (currentFrameStackItem.frameResult.status === "success") {
        frameUiState = {
          status: "complete",
          frame: currentFrameStackItem.frameResult.frame,
          debugImage: enableImageDebugging ? currentFrameStackItem.frameResult.framesDebugInfo?.image : void 0,
          isImageLoading,
          id: currentFrameStackItem.timestamp.getTime(),
          frameState
        };
      } else if (isPartialFrameStackItem(currentFrameStackItem) && allowPartialFrame) {
        frameUiState = {
          status: "partial",
          frame: currentFrameStackItem.frameResult.frame,
          debugImage: enableImageDebugging ? currentFrameStackItem.frameResult.framesDebugInfo?.image : void 0,
          isImageLoading,
          id: currentFrameStackItem.timestamp.getTime(),
          frameState
        };
      } else {
        return components.Error(
          { message: "Invalid frame" },
          theme?.Error || {}
        );
      }
      break;
    }
    case "pending": {
      frameUiState = {
        status: "loading",
        id: currentFrameStackItem.timestamp.getTime(),
        frameState
      };
      break;
    }
  }
  const isLoading = frameUiState.status === "loading" || frameUiState.isImageLoading;
  return components.Root(
    {
      createElement,
      frameState: frameUiState,
      dimensions: isLoading ? rootDimensionsRef.current ?? null : null,
      ref: rootRef,
      loadingScreen: isLoading ? components.LoadingScreen(
        {
          frameState: frameUiState,
          dimensions: rootDimensionsRef.current ?? null
        },
        theme?.LoadingScreen || {}
      ) : null,
      buttonsContainer: frameUiState.status === "loading" || !frameUiState.frame.buttons || frameUiState.frame.buttons.length === 0 ? null : components.ButtonsContainer(
        {
          frameState: frameUiState,
          buttons: frameUiState.frame.buttons.map(
            (frameButton, index) => components.Button(
              {
                frameState: frameUiState,
                frameButton,
                index,
                // @TODO provide previous frame to pending state so we can remove loading check and render some loading indicator?
                isDisabled: false,
                onPress() {
                  rootDimensionsRef.current = rootRef.current?.computeDimensions();
                  Promise.resolve(
                    frameState.onButtonPress(
                      // @todo change the type onButtonPress to accept partial frame as well because that can happen if partial frames are enabled
                      frameUiState.frame,
                      frameButton,
                      index
                    )
                  ).catch((error) => {
                    console.error(error);
                  });
                }
              },
              theme?.Button || {}
            )
          )
        },
        theme?.ButtonsContainer || {}
      ),
      imageContainer: components.ImageContainer(
        {
          frameState: frameUiState,
          aspectRatio: (frameUiState.status === "loading" ? previousFrame?.imageAspectRatio : frameUiState.frame.imageAspectRatio) ?? "1.91:1",
          image: components.Image(
            frameUiState.status === "loading" ? {
              status: "frame-loading",
              frameState: frameUiState,
              onImageLoadEnd,
              aspectRatio: previousFrame?.imageAspectRatio ?? "1.91:1"
            } : {
              status: "frame-loading-complete",
              frameState: frameUiState,
              src: frameUiState.debugImage ?? frameUiState.frame.image,
              aspectRatio: frameUiState.frame.imageAspectRatio ?? "1.91:1",
              onImageLoadEnd
            },
            theme?.Image || {}
          ),
          messageTooltip: currentFrameStackItem.status === "message" ? components.MessageTooltip(
            {
              frameState: frameUiState,
              status: currentFrameStackItem.type === "error" ? "error" : "message",
              message: getErrorMessageFromFramesStackItem(
                currentFrameStackItem
              )
            },
            theme?.MessageTooltip || {}
          ) : null
        },
        theme?.ImageContainer || {}
      ),
      textInputContainer: frameUiState.status === "loading" || typeof frameUiState.frame.inputText !== "string" ? null : components.TextInputContainer(
        {
          frame: frameUiState,
          textInput: components.TextInput(
            {
              // @TODO provide previous frame to pending state so we can remove loading check and render skeletons?
              isDisabled: false,
              frameState: frameUiState,
              placeholder: frameUiState.frame.inputText,
              onChange(text) {
                frameState.setInputText(text);
              },
              value: frameState.inputText
            },
            theme?.TextInput || {}
          )
        },
        theme?.TextInputContainer || {}
      )
    },
    theme?.Root || {}
  );
}

// src/ui/index.tsx
var RootContainer = (0, import_react2.forwardRef)(
  ({
    createElement,
    buttonsContainer,
    imageContainer,
    loadingScreen,
    textInputContainer,
    dimensions,
    className,
    style
  }, ref) => {
    const elementRef = (0, import_react2.useRef)(null);
    (0, import_react2.useImperativeHandle)(ref, () => {
      return {
        computeDimensions() {
          if (!elementRef.current) {
            return void 0;
          }
          const { width, height } = elementRef.current.getBoundingClientRect();
          return { width, height };
        }
      };
    });
    return createElement(
      "div",
      {
        className,
        ref: elementRef,
        style: { ...dimensions, ...style }
      },
      loadingScreen,
      imageContainer,
      textInputContainer,
      buttonsContainer
    );
  }
);
RootContainer.displayName = "RootContainer";
function isCssProperties(value) {
  return typeof value === "object" && value !== null;
}
function createDefaultComponents(createElement = import_react2.createElement) {
  return {
    Button(props, stylingProps) {
      const label = /* @__PURE__ */ import_react2.default.createElement(import_react2.default.Fragment, null, props.frameButton.action === "mint" && "\u2B17 ", props.frameButton.label, props.frameButton.action === "tx" && /* @__PURE__ */ import_react2.default.createElement(
        "svg",
        {
          "aria-hidden": "true",
          focusable: "false",
          role: "img",
          viewBox: "0 0 16 16",
          className: "ml-1 mb-[2px] text-gray-400 inline-block select-none align-text-middle overflow-visible",
          width: "12",
          height: "12",
          fill: "currentColor"
        },
        /* @__PURE__ */ import_react2.default.createElement("path", { d: "M9.504.43a1.516 1.516 0 0 1 2.437 1.713L10.415 5.5h2.123c1.57 0 2.346 1.909 1.22 3.004l-7.34 7.142a1.249 1.249 0 0 1-.871.354h-.302a1.25 1.25 0 0 1-1.157-1.723L5.633 10.5H3.462c-1.57 0-2.346-1.909-1.22-3.004L9.503.429Zm1.047 1.074L3.286 8.571A.25.25 0 0 0 3.462 9H6.75a.75.75 0 0 1 .694 1.034l-1.713 4.188 6.982-6.793A.25.25 0 0 0 12.538 7H9.25a.75.75 0 0 1-.683-1.06l2.008-4.418.003-.006a.036.036 0 0 0-.004-.009l-.006-.006-.008-.001c-.003 0-.006.002-.009.004Z" })
      ), (props.frameButton.action === "post_redirect" || props.frameButton.action === "link") && ` \u2197`);
      return createElement(
        "button",
        {
          ...stylingProps,
          key: props.index,
          disabled: props.isDisabled,
          onClick: props.onPress,
          type: "button"
        },
        label
      );
    },
    ButtonsContainer(props, stylingProps) {
      return createElement("div", stylingProps, props.buttons);
    },
    Error(props, stylingProps) {
      return createElement("div", stylingProps, props.message);
    },
    Image(props, stylingProps) {
      const aspectRatio = props.aspectRatio.replace(":", "/");
      return createElement("img", {
        ...stylingProps,
        "data-aspect-ratio": aspectRatio,
        style: {
          "--frame-image-aspect-ratio": aspectRatio,
          ...isCssProperties(stylingProps.style) && stylingProps.style
        },
        onLoad: props.onImageLoadEnd,
        onError: props.onImageLoadEnd,
        src: props.status === "frame-loading" ? void 0 : props.src,
        alt: "Frame"
      });
    },
    ImageContainer(props, stylingProps) {
      const aspectRatio = props.aspectRatio.replace(":", "/");
      return createElement(
        "div",
        {
          ...stylingProps,
          style: {
            "--frame-image-aspect-ratio": aspectRatio,
            ...isCssProperties(stylingProps.style) && stylingProps.style
          }
        },
        props.image,
        props.messageTooltip
      );
    },
    LoadingScreen(props, stylingProps) {
      return createElement("div", stylingProps, "Loading...");
    },
    Message(props, stylingProps) {
      return createElement(
        "div",
        {
          ...stylingProps,
          "data-status": props.status
        },
        props.message
      );
    },
    MessageTooltip(props, stylingProps) {
      return createElement(
        "div",
        { ...stylingProps, "data-status": props.status },
        props.message
      );
    },
    Root(props, stylingProps) {
      return createElement(RootContainer, { ...props, ...stylingProps });
    },
    TextInput(props, stylingProps) {
      return createElement("input", {
        ...stylingProps,
        disabled: props.isDisabled,
        onChange: (e) => {
          props.onChange(e.target.value);
        },
        placeholder: props.placeholder,
        value: props.value,
        type: "text"
      });
    },
    TextInputContainer(props, stylingProps) {
      return createElement("div", stylingProps, props.textInput);
    }
  };
}
function FrameUI(props) {
  const defaultComponents = (0, import_react2.useMemo)(
    () => createDefaultComponents(props.createElement),
    [props.createElement]
  );
  const components = (0, import_react2.useMemo)(() => {
    return {
      ...defaultComponents,
      ...props.components
    };
  }, [defaultComponents, props.components]);
  return /* @__PURE__ */ import_react2.default.createElement(BaseFrameUI, { ...props, components });
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  FrameUI
});
