import {
  BaseFrameUI
} from "../chunk-B22B4XZC.js";
import "../chunk-NKAZJQ23.js";
import "../chunk-6C3VEZWH.js";

// src/ui/index.native.tsx
import React, {
  createElement as reactCreateElement,
  forwardRef,
  useImperativeHandle,
  useMemo,
  useRef
} from "react";
import {
  Button,
  Image,
  Text,
  TextInput,
  View
} from "react-native";
var RootContainer = forwardRef(
  ({
    createElement,
    buttonsContainer,
    imageContainer,
    loadingScreen,
    textInputContainer,
    dimensions,
    className,
    style
  }, ref) => {
    const viewDimensionsRef = useRef();
    useImperativeHandle(ref, () => {
      return {
        computeDimensions() {
          return viewDimensionsRef.current;
        }
      };
    });
    return createElement(
      View,
      {
        // @ts-expect-error -- this is supported if nativewind is used
        className,
        onLayout: (e) => {
          viewDimensionsRef.current = {
            height: e.nativeEvent.layout.height,
            width: e.nativeEvent.layout.width
          };
        },
        style: {
          ...dimensions,
          ...style
        }
      },
      loadingScreen,
      imageContainer,
      textInputContainer,
      buttonsContainer
    );
  }
);
RootContainer.displayName = "RootContainer";
function isStyleProp(value) {
  return typeof value === "object" && value !== null;
}
function createDefaultComponents(createElement = reactCreateElement) {
  return {
    Button(props, stylingProps) {
      return createElement(Button, {
        ...stylingProps,
        disabled: props.isDisabled,
        onPress: props.onPress,
        title: props.frameButton.label
      });
    },
    ButtonsContainer(props, stylingProps) {
      return createElement(View, stylingProps, props.buttons);
    },
    Error(props, stylingProps) {
      return createElement(
        View,
        stylingProps,
        createElement(Text, null, props.message)
      );
    },
    Image(props, stylingProps) {
      const aspectRatio = props.aspectRatio.replace(":", "/");
      if (props.status === "frame-loading") {
        return createElement(View);
      }
      return createElement(Image, {
        ...stylingProps,
        style: {
          aspectRatio,
          ...isStyleProp(stylingProps.style) && stylingProps.style
        },
        src: props.src,
        alt: "Frame"
      });
    },
    ImageContainer(props, stylingProps) {
      const aspectRatio = props.aspectRatio.replace(":", "/");
      return createElement(
        View,
        {
          ...stylingProps,
          style: {
            aspectRatio,
            ...isStyleProp(stylingProps.style) && stylingProps.style
          }
        },
        props.image,
        props.messageTooltip
      );
    },
    LoadingScreen(props, stylingProps) {
      return createElement(
        View,
        stylingProps,
        createElement(Text, null, "Loading...")
      );
    },
    Message(props, stylingProps) {
      return createElement(
        View,
        stylingProps,
        createElement(Text, null, props.message)
      );
    },
    MessageTooltip(props, stylingProps) {
      return createElement(
        View,
        stylingProps,
        createElement(Text, null, props.message)
      );
    },
    Root(props, stylingProps) {
      return createElement(RootContainer, {
        ...props,
        ...stylingProps,
        createElement
      });
    },
    TextInput(props, stylingProps) {
      return createElement(TextInput, {
        ...stylingProps,
        editable: !props.isDisabled,
        onChangeText: props.onChange,
        placeholder: props.placeholder,
        value: props.value
      });
    },
    TextInputContainer(props, stylingProps) {
      return createElement(View, stylingProps, props.textInput);
    }
  };
}
function FrameUI(props) {
  const defaultComponents = useMemo(
    () => createDefaultComponents(props.createElement),
    [props.createElement]
  );
  const components = useMemo(() => {
    return {
      ...defaultComponents,
      ...props.components
    };
  }, [defaultComponents, props.components]);
  return /* @__PURE__ */ React.createElement(BaseFrameUI, { ...props, components });
}
export {
  FrameUI
};
