"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/use-frame-stack.ts
var use_frame_stack_exports = {};
__export(use_frame_stack_exports, {
  isParseResult: () => isParseResult,
  useFrameStack: () => useFrameStack
});
module.exports = __toCommonJS(use_frame_stack_exports);
var import_react = require("react");
function computeDurationInSeconds(start, end) {
  return Number(((end.getTime() - start.getTime()) / 1e3).toFixed(2));
}
function isParseResult(result) {
  return typeof result === "object" && result !== null && "status" in result;
}
function framesStackReducer(state, action) {
  switch (action.action) {
    case "LOAD":
      return [action.item, ...state];
    case "DONE_REDIRECT": {
      const index = state.findIndex(
        (item) => item.timestamp === action.pendingItem.timestamp
      );
      if (index === -1) {
        return state;
      }
      state[index] = {
        ...action.pendingItem,
        ...action.item,
        status: "doneRedirect"
      };
      return state.slice();
    }
    case "DONE":
    case "REQUEST_ERROR": {
      const index = state.findIndex(
        (item) => item.timestamp === action.pendingItem.timestamp
      );
      if (index === -1) {
        return state;
      }
      state[index] = action.item;
      return state.slice();
    }
    case "RESET_INITIAL_FRAME": {
      const originalInitialFrame = state[0];
      const frame = isParseResult(action.resultOrFrame) ? action.resultOrFrame.frame : action.resultOrFrame;
      const shouldReset = !originalInitialFrame || originalInitialFrame.status === "done" && originalInitialFrame.frameResult.frame !== frame;
      if (shouldReset) {
        const frameResult = isParseResult(action.resultOrFrame) ? action.resultOrFrame : {
          status: "success",
          reports: {},
          frame: action.resultOrFrame
        };
        return [
          {
            request: {
              method: "GET",
              url: action.homeframeUrl ?? ""
            },
            url: action.homeframeUrl ?? "",
            requestDetails: {},
            response: new Response(JSON.stringify(frameResult), {
              status: 200,
              headers: { "Content-Type": "application/json" }
            }),
            responseStatus: 200,
            timestamp: /* @__PURE__ */ new Date(),
            speed: 0,
            frameResult,
            status: "done",
            // @todo should this be result or frame?
            responseBody: frameResult
          }
        ];
      }
      return state;
    }
    case "CLEAR":
      return [];
    default:
      return state;
  }
}
function useFrameStack({
  initialFrame,
  initialFrameUrl
}) {
  const [stack, dispatch] = (0, import_react.useReducer)(
    framesStackReducer,
    [initialFrame, initialFrameUrl],
    ([frame, frameUrl]) => {
      if (frame) {
        const frameResult = isParseResult(frame) ? frame : {
          reports: {},
          frame,
          status: "success"
        };
        return [
          {
            response: new Response(JSON.stringify(frameResult), {
              status: 200,
              headers: { "Content-Type": "application/json" }
            }),
            responseStatus: 200,
            responseBody: frameResult,
            timestamp: /* @__PURE__ */ new Date(),
            requestDetails: {},
            request: {
              method: "GET",
              url: frameUrl ?? ""
            },
            speed: 0,
            frameResult,
            status: "done",
            url: frameUrl ?? ""
          }
        ];
      } else if (frameUrl) {
        return [
          {
            method: "GET",
            request: {
              method: "GET",
              url: frameUrl
            },
            url: frameUrl,
            requestDetails: {},
            timestamp: /* @__PURE__ */ new Date(),
            status: "pending"
          }
        ];
      }
      return [];
    }
  );
  const api = (0, import_react.useMemo)(() => {
    return {
      clear() {
        dispatch({
          action: "CLEAR"
        });
      },
      createGetPendingItem(arg) {
        const item = {
          method: "GET",
          request: arg.request,
          requestDetails: {},
          url: arg.request.url,
          timestamp: /* @__PURE__ */ new Date(),
          status: "pending"
        };
        dispatch({
          action: "LOAD",
          item
        });
        return item;
      },
      createPostPendingItem(arg) {
        const item = {
          method: "POST",
          request: arg.request,
          requestDetails: {
            body: arg.action.body,
            searchParams: arg.action.searchParams
          },
          url: arg.action.searchParams.get("postUrl") ?? "missing postUrl",
          timestamp: /* @__PURE__ */ new Date(),
          status: "pending"
        };
        dispatch({
          action: "LOAD",
          item
        });
        return item;
      },
      createCastOrComposerActionPendingItem(arg) {
        return {
          method: "POST",
          requestDetails: {
            body: arg.action.body,
            searchParams: arg.action.searchParams
          },
          request: arg.request,
          status: "pending",
          timestamp: /* @__PURE__ */ new Date(),
          url: arg.action.searchParams.get("postUrl") ?? "missing postUrl"
        };
      },
      markCastFrameAsDone() {
      },
      markCastMessageAsDone(arg) {
        dispatch({
          action: "LOAD",
          item: arg.pendingItem
        });
        dispatch({
          action: "DONE",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "message",
            message: arg.responseData.message,
            response: arg.response.clone(),
            responseBody: arg.responseData,
            responseStatus: arg.response.status,
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            ),
            type: "info"
          }
        });
      },
      markComposerFormActionAsDone() {
      },
      markAsDone(arg) {
        dispatch({
          action: "DONE",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "done",
            frameResult: arg.frameResult,
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            ),
            response: arg.response.clone(),
            responseStatus: arg.response.status,
            responseBody: arg.frameResult
          }
        });
      },
      markAsDoneWithErrorMessage(arg) {
        dispatch({
          action: "DONE",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            responseStatus: arg.response.status,
            response: arg.response.clone(),
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            ),
            status: "message",
            type: "error",
            message: arg.responseData.message,
            responseBody: arg.responseData
          }
        });
      },
      markAsDoneWithRedirect(arg) {
        dispatch({
          action: "DONE_REDIRECT",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            location: arg.location,
            response: arg.response.clone(),
            responseBody: arg.responseBody,
            responseStatus: arg.response.status,
            status: "doneRedirect",
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            )
          }
        });
      },
      markAsFailed(arg) {
        dispatch({
          action: "REQUEST_ERROR",
          pendingItem: arg.pendingItem,
          item: {
            request: arg.pendingItem.request,
            requestDetails: arg.pendingItem.requestDetails,
            timestamp: arg.pendingItem.timestamp,
            url: arg.pendingItem.url,
            response: arg.response?.clone() ?? null,
            responseStatus: arg.responseStatus,
            requestError: arg.requestError,
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            ),
            status: "requestError",
            responseBody: arg.responseBody
          }
        });
      },
      markAsFailedWithRequestError(arg) {
        dispatch({
          action: "REQUEST_ERROR",
          pendingItem: arg.pendingItem,
          item: {
            ...arg.pendingItem,
            status: "requestError",
            requestError: arg.error,
            response: arg.response.clone(),
            responseStatus: arg.response.status,
            responseBody: arg.responseBody,
            speed: computeDurationInSeconds(
              arg.pendingItem.timestamp,
              arg.endTime
            )
          }
        });
      }
    };
  }, [dispatch]);
  return [stack, dispatch, api];
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  isParseResult,
  useFrameStack
});
