import "./chunk-45S4JHLU.js";
import "./chunk-RJBA7PM4.js";
import {
  unsignedFrameAction
} from "./chunk-KLEIAKQW.js";
import "./chunk-3CCTWQZS.js";
import {
  useFetchFrame
} from "./chunk-CKALV6XW.js";
import {
  useFrameStack
} from "./chunk-B237QKYM.js";
import "./chunk-6C3VEZWH.js";

// src/use-frame.tsx
import { useCallback, useEffect, useMemo, useRef, useState } from "react";
function onMintFallback({ target }) {
  console.log("Please provide your own onMint function to useFrame() hook.");
  const message = `Mint requested: ${target}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
}
function onConnectWalletFallback() {
  throw new Error(
    "Please implement this function in order to use transactions"
  );
}
async function onTransactionFallback({
  transactionData
}) {
  console.log(
    "Please provide your own onTransaction function to useFrame() hook."
  );
  const message = `Requesting a transaction on chain with ID ${transactionData.chainId} with the following params: ${JSON.stringify(
    transactionData.params,
    null,
    2
  )}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
  return null;
}
async function onSignatureFallback({
  signatureData
}) {
  console.log(
    "Please provide your own onSignature function to useFrame() hook."
  );
  const message = `Requesting a signature on chain with ID ${signatureData.chainId} with the following params: ${JSON.stringify(
    signatureData.params,
    null,
    2
  )}`;
  if (typeof window !== "undefined") {
    window.alert(message);
  } else {
    console.log(message);
  }
  return null;
}
function handleRedirectFallback(location) {
  console.log(
    "Please provide your own onRedirect function to useFetchFrame() hook."
  );
  const message = `You are about to be redirected to ${location.toString()}`;
  if (typeof window !== "undefined") {
    if (window.confirm(message)) {
      window.open(location, "_blank")?.focus();
    }
  } else {
    console.log(message);
  }
}
function handleLinkButtonClickFallback(button) {
  console.log(
    "Please provide your own onLinkButtonClick function to useFrame() hook."
  );
  if (typeof window !== "undefined") {
    if (window.confirm(`You are about to be redirected to ${button.target}`)) {
      parent.window.open(button.target, "_blank");
    }
  } else {
    console.log(`Link button with target ${button.target} clicked.`);
  }
}
function defaultComposerFormActionHandler() {
  throw new Error('Please implement your own "onComposerFormAction" handler');
}
function useFrame({
  homeframeUrl,
  frameContext,
  dangerousSkipSigning,
  onMint = onMintFallback,
  onTransaction = onTransactionFallback,
  transactionDataSuffix,
  onConnectWallet = onConnectWalletFallback,
  onSignature = onSignatureFallback,
  connectedAddress,
  signerState,
  frame,
  /** Ex: /frames */
  frameActionProxy,
  /** Ex: /frames */
  frameGetProxy,
  extraButtonRequestPayload,
  specification = "farcaster",
  onError,
  onLinkButtonClick = handleLinkButtonClickFallback,
  onRedirect = handleRedirectFallback,
  fetchFn = (...args) => fetch(...args),
  onComposerFormAction = defaultComposerFormActionHandler
}) {
  const [inputText, setInputText] = useState("");
  const [framesStack, dispatch, stackAPI] = useFrameStack({
    initialFrame: frame,
    initialFrameUrl: homeframeUrl
  });
  const fetchFrame = useFetchFrame({
    stackAPI,
    frameActionProxy,
    frameGetProxy,
    onTransaction,
    transactionDataSuffix,
    onSignature,
    signFrameAction({ actionContext, forceRealSigner }) {
      return dangerousSkipSigning && !forceRealSigner ? unsignedFrameAction(actionContext) : signerState.signFrameAction(actionContext);
    },
    specification,
    stackDispatch: dispatch,
    extraButtonRequestPayload,
    onError,
    fetchFn,
    onRedirect,
    onComposerFormAction
  });
  const fetchFrameRef = useRef(fetchFrame);
  fetchFrameRef.current = fetchFrame;
  const onErrorRef = useRef(onError);
  onErrorRef.current = onError;
  useEffect(() => {
    if (!frame && homeframeUrl) {
      fetchFrameRef.current(
        {
          url: homeframeUrl,
          method: "GET"
        },
        // tell the fetchFrame function to clear the stack because this is called only on initial render
        // and there could potentially be a pending object returned from SSR
        true
      ).catch((e) => {
        console.error(e);
      });
    } else if (frame) {
      dispatch({
        action: "RESET_INITIAL_FRAME",
        resultOrFrame: frame,
        homeframeUrl
      });
    }
  }, [frame, homeframeUrl, dispatch]);
  const onPostButton = useCallback(
    async function onPostButton2({
      currentFrame,
      buttonIndex,
      postInputText,
      frameButton,
      target,
      state,
      fetchFrameOverride
    }) {
      if (!dangerousSkipSigning && !signerState.hasSigner) {
        const error = new Error("Missing signer");
        onErrorRef.current?.(error);
        console.error(`@frames.js/render: ${error.message}`);
        return;
      }
      if (!homeframeUrl) {
        const error = new Error("Missing homeframeUrl");
        onErrorRef.current?.(error);
        console.error(`@frames.js/render: ${error.message}`);
        return;
      }
      const _fetchFrame = fetchFrameOverride ?? fetchFrame;
      await _fetchFrame({
        frameButton,
        isDangerousSkipSigning: dangerousSkipSigning ?? false,
        method: "POST",
        signerStateActionContext: {
          inputText: postInputText,
          signer: signerState.signer ?? null,
          frameContext,
          url: homeframeUrl,
          target,
          frameButton,
          buttonIndex,
          state
        },
        sourceFrame: currentFrame
      });
    },
    [
      dangerousSkipSigning,
      fetchFrame,
      frameContext,
      homeframeUrl,
      signerState.hasSigner,
      signerState.signer
    ]
  );
  const onConnectWalletRef = useRef(onConnectWallet);
  onConnectWalletRef.current = onConnectWallet;
  const onTransactionButton = useCallback(
    async function onTransactionButton2({
      currentFrame,
      buttonIndex,
      postInputText,
      frameButton
    }) {
      if (!dangerousSkipSigning && !signerState.hasSigner) {
        const error = new Error("Missing signer");
        onErrorRef.current?.(error);
        console.error(`@frames.js/render: ${error.message}`);
        return;
      }
      if (!homeframeUrl) {
        const error = new Error("Missing homeframeUrl");
        onErrorRef.current?.(error);
        console.error(`@frames.js/render: ${error.message}`);
        return;
      }
      if (!connectedAddress) {
        try {
          onConnectWalletRef.current();
        } catch (e) {
          onErrorRef.current?.(e instanceof Error ? e : new Error(String(e)));
          console.error(`@frames.js/render: ${String(e)}`);
        }
        return;
      }
      await fetchFrame({
        frameButton,
        isDangerousSkipSigning: dangerousSkipSigning ?? false,
        method: "POST",
        signerStateActionContext: {
          type: "tx-data",
          inputText: postInputText,
          signer: signerState.signer ?? null,
          frameContext,
          address: connectedAddress,
          url: homeframeUrl,
          target: frameButton.target,
          frameButton,
          buttonIndex,
          state: currentFrame.state
        },
        sourceFrame: currentFrame
      });
    },
    [
      fetchFrame,
      dangerousSkipSigning,
      frameContext,
      connectedAddress,
      homeframeUrl,
      signerState
    ]
  );
  const onButtonPress = useCallback(
    async function onButtonPress2(currentFrame, frameButton, index, fetchFrameOverride = fetchFrame) {
      const clientSideActions = ["mint", "link"];
      const buttonRequiresAuth = !clientSideActions.includes(
        frameButton.action
      );
      if (!signerState.hasSigner && !dangerousSkipSigning && buttonRequiresAuth) {
        await signerState.onSignerlessFramePress();
        return;
      }
      switch (frameButton.action) {
        case "link": {
          onLinkButtonClick(frameButton);
          break;
        }
        case "mint": {
          onMint({
            frameButton,
            target: frameButton.target,
            frame: currentFrame
          });
          break;
        }
        case "tx": {
          await onTransactionButton({
            frameButton,
            buttonIndex: index + 1,
            postInputText: currentFrame.inputText !== void 0 ? inputText : void 0,
            currentFrame
          });
          break;
        }
        case "post":
        case "post_redirect": {
          try {
            const target = frameButton.target || frameButton.post_url || currentFrame.postUrl || homeframeUrl;
            if (!target) {
              throw new Error("missing target");
            }
            await onPostButton({
              currentFrame,
              frameButton,
              /** https://docs.farcaster.xyz/reference/frames/spec#handling-clicks
              
                      POST the packet to fc:frame:button:$idx:action:target if present
                      POST the packet to fc:frame:post_url if target was not present.
                      POST the packet to or the frame's embed URL if neither target nor action were present.
                      */
              target,
              buttonIndex: index + 1,
              postInputText: currentFrame.inputText !== void 0 ? inputText : void 0,
              state: currentFrame.state,
              fetchFrameOverride
            });
            setInputText("");
          } catch (err) {
            if (err instanceof Error) {
              onErrorRef.current?.(err);
            }
            console.error(err);
          }
          break;
        }
        default:
          throw new Error("Unrecognized frame button action");
      }
    },
    [
      dangerousSkipSigning,
      fetchFrame,
      homeframeUrl,
      inputText,
      onLinkButtonClick,
      onMint,
      onPostButton,
      onTransactionButton,
      signerState
    ]
  );
  const clearFrameStack = useCallback(() => {
    dispatch({ action: "CLEAR" });
  }, [dispatch]);
  const onCastActionButtonPress = useCallback(
    async function onActionButtonPress(arg) {
      if (!signerState.hasSigner && !dangerousSkipSigning) {
        await signerState.onSignerlessFramePress();
        return;
      }
      return fetchFrame(
        {
          method: "CAST_ACTION",
          action: arg.castAction,
          isDangerousSkipSigning: dangerousSkipSigning ?? false,
          signerStateActionContext: {
            signer: signerState.signer ?? null,
            frameContext,
            url: arg.castAction.url,
            target: arg.castAction.url,
            buttonIndex: 1
          }
        },
        arg.clearStack
      );
    },
    [dangerousSkipSigning, fetchFrame, frameContext, signerState]
  );
  const onComposerActionButtonPress = useCallback(
    async function onActionButtonPress(arg) {
      if (!signerState.hasSigner && !dangerousSkipSigning) {
        await signerState.onSignerlessFramePress();
        return;
      }
      return fetchFrame(
        {
          method: "COMPOSER_ACTION",
          action: arg.castAction,
          isDangerousSkipSigning: dangerousSkipSigning ?? false,
          composerActionState: arg.composerActionState,
          signerStateActionContext: {
            signer: signerState.signer ?? null,
            frameContext,
            url: arg.castAction.url,
            target: arg.castAction.url,
            buttonIndex: 1
          }
        },
        arg.clearStack
      );
    },
    [dangerousSkipSigning, fetchFrame, frameContext, signerState]
  );
  return useMemo(() => {
    return {
      inputText,
      setInputText,
      clearFrameStack,
      dispatchFrameStack: dispatch,
      onButtonPress,
      fetchFrame,
      homeframeUrl,
      framesStack,
      currentFrameStackItem: framesStack[0],
      onCastActionButtonPress,
      onComposerActionButtonPress
    };
  }, [
    inputText,
    clearFrameStack,
    dispatch,
    onButtonPress,
    fetchFrame,
    homeframeUrl,
    framesStack,
    onCastActionButtonPress,
    onComposerActionButtonPress
  ]);
}
export {
  useFrame
};
